program app;

{$mode objfpc}
{$modeswitch externalclass}

uses
  browserconsole, browserapp, JS, Classes, SysUtils, Web, WebAssembly, types,
  wasienv, wasihostapp;

function init_state : double;

begin
  Console.Log('Init state');
  Result:=2.71;
end;

var
  deltacount : integer = 0;

function compute_delta : TJSPromise;

  function dotext (aValue : JSValue) : JSValue;

  var
    txt : String absolute avalue;

  begin
    result:=parseFloat(txt);
  end;

  function dores (aValue : JSValue) : JSValue;

  var
    Res : TJSResponse absolute aValue;

  begin
    Result:=Res.text
  end;

begin
  inc(DeltaCount);
  Console.Log('compute delta ',Deltacount);
  Result:=window.fetch('data.txt')
         ._then(@dores)
         ._then(@DoText);
end;

Type

  { TExampleExtension }

  TExampleExtension = Class(TImportExtension)
    Procedure FillImportObject(aObject: TJSObject); override;
    Function ImportName: String; override;
  end;

  { TMyApplication }

  TMyApplication = class(TWASIHostApplication)
  Private
    procedure DoWrite(Sender: TObject; const aOutput: String);
  Public
    Constructor Create(aOwner : TComponent); override;
    procedure doRun; override;
  end;

  TWebAssemblyFunctionType = class external name 'Object' (TJSObject)
    parameters : TJSArray;
    results : TJSObject;
  end;

  TWebAssemblyFunctionOptions = class external name 'Object' (TJSObject)
    suspending : string;
  end;


  TWebAssemblyFunction = Class external name 'WebAssembly.Function' (TJSFunction)
    constructor new(atype: TJSObject; aFunc : TJSFunction; options : TJSObject);
    constructor new(atype: TWebAssemblyFunctionType; aFunc : TJSFunction; options : TWebAssemblyFunctionOptions);
  end;

{ TExampleExtension }


procedure TExampleExtension.FillImportObject(aObject: TJSObject);

var
  suspending_compute_delta : TWebAssemblyFunction;
  aType,aOpts : TJSObject;

begin
  aType:=new(['parameters', TJSArray.New('externref'), 'results',TJSArray.New('f64')]);
  aOpts:=new(['suspending','first']);
  suspending_compute_delta :=TWebAssemblyFunction.New(aType,TJSFunction(@compute_delta),aOpts);
  aObject['compute_delta']:=suspending_compute_delta;
  aObject['init_state']:=@init_state;
end;

function TExampleExtension.ImportName: String;
begin
  Result:='js'
end;


procedure TMyApplication.DoWrite(Sender: TObject; const aOutput: String);
begin
  Writeln(aOutput);
end;

constructor TMyApplication.Create(aOwner: TComponent);

var
  Ext : TExampleExtension;

begin
  inherited Create(aOwner);
  WasiEnvironment.OnStdErrorWrite:=@DoWrite;
  WasiEnvironment.OnStdOutputWrite:=@DoWrite;
  // export our function
  Ext:=TExampleExtension.Create(WasiEnvironment);
end;

procedure TMyApplication.doRun;

  procedure DoStart(Sender: TObject; aDescriptor: TWebAssemblyStartDescriptor);

  function MakeResolver(b : integer) : TJSPromiseResolver;

  begin
    Result:= function (a : jsvalue) : jsvalue
    begin
      Writeln(b,' : ',Double(a));
    end;
  end;

  type
    TUpdateFunc = function : TJSPromise;

  Var
    update_state : TWebAssemblyFunction;
    res : TJSPromiseResolver;
    I : integer;

  begin
    update_state:=TWebAssemblyFunction.New(
      New(['parameters',TJSArray.New(), 'results', TJSArray.New('externref')]),
      exported['update_state'],
      New(['promising','first'])
    );
    exported.initialize;
    for I:=1 to 10 do
      begin
      Res:=MakeResolver(I);
      TUpdateFunc(Update_State)()._then(res);
      end;
  end;

var
  P : TJSPromise;

begin
  Terminate;
//  P:=StartWebAssembly('tjspromise1.wasm',True,Nil,@DoStart);
  P:=StartWebAssembly('tjspromise1.wasm',false,Nil,@DoStart);
  P._then(function(a : JSValue) : JSValue
  begin
    console.debug(a);
    DoStart(Self,TWebAssemblyStartDescriptor(a));
  end);
end;

var
  Application : TMyApplication;

begin
  Application:=TMyApplication.Create(nil);
  Application.Initialize;
  Application.Run;
end.
